==> yawp-1.20.1-fabric-0.6.2-beta1-sources.jar: de/z0rdak/yawp/data/region/LevelRegionData.java <==
package de.z0rdak.yawp.data.region;

import com.mojang.serialization.Codec;
import com.mojang.serialization.Lifecycle;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import de.z0rdak.yawp.api.commands.CommandConstants;
import de.z0rdak.yawp.api.core.RegionManager;
import de.z0rdak.yawp.commands.arguments.region.RegionArgumentType;
import de.z0rdak.yawp.constants.Constants;
import de.z0rdak.yawp.core.region.*;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.*;
import java.util.function.Supplier;
import java.util.stream.Collectors;
import net.minecraft.class_18;
import net.minecraft.class_1937;
import net.minecraft.class_2487;
import net.minecraft.class_2509;
import net.minecraft.class_2520;
import net.minecraft.class_26;
import net.minecraft.class_2960;
import net.minecraft.class_5321;
import net.minecraft.class_7924;

import static de.z0rdak.yawp.api.commands.CommandConstants.values;

public class LevelRegionData extends class_18 {

    public static Codec<LevelRegionData> CODEC = RecordCodecBuilder.create(
            instance -> instance.group(
                    class_2960.field_25139.fieldOf("id")
                            .forGetter(lrd -> lrd.id),
                    DimensionalRegion.CODEC.fieldOf("dim_region")
                            .forGetter(lrd -> lrd.dim),
                    Codec.unboundedMap(Codec.STRING, MarkedRegion.CODEC).optionalFieldOf("local_regions", Lifecycle.stable(), new HashMap<>(), Lifecycle.stable())
                            .forGetter(lrd -> lrd.locals)
            ).apply(instance, LevelRegionData::new));

    public static String buildSavedDataType(class_2960 dimRl) {
        var dimId = dimRl.toString().replace(class_2960.field_33380, '_');
        return String.join("/", Constants.MOD_ID, dimId);
    }

    public static LevelRegionData get(class_26 storage, class_2960 dimRl, @Nullable Supplier<LevelRegionData> defaultSupplier) {
        String levelFileLoc = LevelRegionData.buildSavedDataType(dimRl);
        return storage.method_17924(
                LevelRegionData::load,
                defaultSupplier == null ? () -> new LevelRegionData(dimRl) : defaultSupplier,
                levelFileLoc
        );
    }

    @Override
    public @NotNull class_2487 method_75(@NotNull class_2487 tag) {
        Optional<class_2520> nbt = LevelRegionData.CODEC.encodeStart(class_2509.field_11560, this)
                .resultOrPartial(Constants.LOGGER::warn);
        if (nbt.isPresent()) {
            tag = (class_2487) nbt.get();
        }
        return tag;
    }

    @Nullable
    public static LevelRegionData load(class_2487 tag) {
        return LevelRegionData.CODEC.parse(class_2509.field_11560, tag)
                .resultOrPartial(Constants.LOGGER::warn)
                .orElse(null);
    }

    private final class_2960 id;
    private  HashMap<String, IMarkableRegion> locals;
    private DimensionalRegion dim;

    public LevelRegionData(class_2960 id, DimensionalRegion dim, Map<String, IMarkableRegion> locals) {
        this(id);
        this.dim = dim;
        this.locals.putAll(locals);
    }

    public LevelRegionData(class_2960 id, DimensionalRegion dim) {
        this(id);
        this.dim = dim;
        this.locals = new HashMap<>();
    }

    public LevelRegionData(class_2960 id) {
        this.id = id;
        this.locals = new HashMap<>();
        var global = RegionManager.get().getGlobalRegion();
        class_5321<class_1937> levelRk = class_5321.method_29179(class_7924.field_41223, id);
        this.dim = new DimensionalRegion(levelRk, global);
    }

    public HashMap<String, IMarkableRegion> getLocals() {
        return locals;
    }

    public class_2960 getId() {
        return id;
    }

    public class_5321<class_1937> getDimKey() {
        return class_5321.method_29179(class_7924.field_41223, this.getId());
    }

    public int regionCount() {
        return locals.size();
    }

    public Collection<IMarkableRegion> getLocalList() {
        return locals.values().stream().toList();
    }

    public Collection<String> getLocalNames() {
        return locals.keySet().stream().toList();
    }

    public DimensionalRegion getDim() {
        return dim;
    }

    public void renameLocal(IMarkableRegion region, String regionName) {
        if (this.locals.containsKey(regionName)) {
            throw new IllegalArgumentException("Region with name '" + regionName + "' already exists in dimension '" + this.dim.getName() + "'!");
        }
        IMarkableRegion currentRegion = this.locals.get(region.getName());
        IProtectedRegion parent = currentRegion.getParent();
        this.removeLocal(currentRegion);
        currentRegion.rename(regionName);
        this.addLocal(parent, currentRegion);
    }

    /**
     * Method to check if a region name is valid for a given dimension. <br>
     * A region name is valid if it matches the pattern and is not already used in the dimension.
     *
     * @param regionName the name of the region to be checked.
     * @return -1 if the region name is invalid, 0 if the region name is valid, 1 if the region name is already used in the dimension.
     */
    public int isValidRegionName(String regionName) {
        List<String> commandStrings = Arrays.stream(values()).map(CommandConstants::toString).collect(Collectors.toList());
        if (!regionName.matches(RegionArgumentType.VALID_NAME_PATTERN.pattern())
                || commandStrings.contains(regionName.toLowerCase())) {
            return -1;
        }
        if (this.locals.containsKey(regionName)) {
            return 1;
        }
        return 0;
    }

    public void addLocal(IProtectedRegion parent, IMarkableRegion child) {
        parent.addChild(child);
        this.locals.put(child.getName(), child);
    }

    public void addLocal(IMarkableRegion child) {
        this.dim.addChild(child);
        this.locals.put(child.getName(), child);
    }

    public void removeLocal(IMarkableRegion region) {
        if (this.hasLocal(region.getName())) {
            this.locals.remove(region.getName());
            if (region.getParent().getRegionType() == RegionType.DIMENSION) {
                region.getParent().removeChild(region);
            }
        }
    }

    public boolean hasLocal(String regionName) {
        return locals.containsKey(regionName);
    }

    public IMarkableRegion getLocal(String regionName) {
        return locals.get(regionName);
    }

    public void clearLocals() {
        this.locals.clear();
        this.dim.clearChildren();
    }

}

==> yawp-1.21.4-fabric-0.6.1-beta1-sources.jar: de/z0rdak/yawp/data/region/LevelRegionData.java <==
package de.z0rdak.yawp.data.region;

import com.mojang.serialization.Codec;
import com.mojang.serialization.Lifecycle;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import de.z0rdak.yawp.api.commands.CommandConstants;
import de.z0rdak.yawp.api.core.RegionManager;
import de.z0rdak.yawp.commands.arguments.region.RegionArgumentType;
import de.z0rdak.yawp.constants.Constants;
import de.z0rdak.yawp.core.region.*;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.*;
import java.util.function.Supplier;
import java.util.stream.Collectors;
import net.minecraft.class_18;
import net.minecraft.class_1937;
import net.minecraft.class_2487;
import net.minecraft.class_2509;
import net.minecraft.class_2520;
import net.minecraft.class_26;
import net.minecraft.class_2960;
import net.minecraft.class_4284;
import net.minecraft.class_5321;
import net.minecraft.class_7225;
import net.minecraft.class_7924;

import static de.z0rdak.yawp.api.commands.CommandConstants.values;

public class LevelRegionData extends class_18 {

    public static Codec<LevelRegionData> CODEC = RecordCodecBuilder.create(
            instance -> instance.group(
                    class_2960.field_25139.fieldOf("id")
                            .forGetter(lrd -> lrd.id),
                    DimensionalRegion.CODEC.fieldOf("dim_region")
                            .forGetter(lrd -> lrd.dim),
                    Codec.unboundedMap(Codec.STRING, MarkedRegion.CODEC).optionalFieldOf("local_regions", Lifecycle.stable(), new HashMap<>(), Lifecycle.stable())
                            .forGetter(lrd -> lrd.locals)
            ).apply(instance, LevelRegionData::new));

    public static String buildSavedDataType(class_2960 dimRl) {
        var dimId = dimRl.toString().replace(class_2960.field_33380, '_');
        return String.join("/", Constants.MOD_ID, dimId);
    }

    public static LevelRegionData get(class_26 storage, class_2960 dimRl, @Nullable Supplier<LevelRegionData> defaultSupplier) {
        String levelFileLoc = LevelRegionData.buildSavedDataType(dimRl);
        Supplier<LevelRegionData> supplier = defaultSupplier == null ? () -> new LevelRegionData(dimRl) : defaultSupplier;
        class_8645<LevelRegionData> factory = new class_8645<>(supplier, LevelRegionData::load, class_4284.field_45079);
        return storage.method_17924(factory, levelFileLoc);
    }

    @Override
    public @NotNull class_2487 method_75(@NotNull class_2487 tag, class_7225.class_7874 provider) {
        Optional<class_2520> nbt = LevelRegionData.CODEC.encodeStart(class_2509.field_11560, this)
                .resultOrPartial(Constants.LOGGER::warn);
        if (nbt.isPresent()) {
            tag = (class_2487) nbt.get();
        }
        return tag;
    }

    @Nullable
    public static LevelRegionData load(class_2487 tag, class_7225.class_7874 provider) {
        return LevelRegionData.CODEC.parse(class_2509.field_11560, tag)
                .resultOrPartial(Constants.LOGGER::warn)
                .orElse(null);
    }

    private final class_2960 id;
    private  HashMap<String, IMarkableRegion> locals;
    private DimensionalRegion dim;

    public LevelRegionData(class_2960 id, DimensionalRegion dim, Map<String, IMarkableRegion> locals) {
        this(id);
        this.dim = dim;
        this.locals.putAll(locals);
    }

    public LevelRegionData(class_2960 id, DimensionalRegion dim) {
        this(id);
        this.dim = dim;
        this.locals = new HashMap<>();
    }

    public LevelRegionData(class_2960 id) {
        this.id = id;
        this.locals = new HashMap<>();
        var global = RegionManager.get().getGlobalRegion();
        class_5321<class_1937> levelRk = class_5321.method_29179(class_7924.field_41223, id);
        this.dim = new DimensionalRegion(levelRk, global);
    }

    public HashMap<String, IMarkableRegion> getLocals() {
        return locals;
    }

    public class_2960 getId() {
        return id;
    }

    public class_5321<class_1937> getDimKey() {
        return class_5321.method_29179(class_7924.field_41223, this.getId());
    }

    public int regionCount() {
        return locals.size();
    }

    public Collection<IMarkableRegion> getLocalList() {
        return locals.values().stream().toList();
    }

    public Collection<String> getLocalNames() {
        return locals.keySet().stream().toList();
    }

    public DimensionalRegion getDim() {
        return dim;
    }

    public void renameLocal(IMarkableRegion region, String regionName) {
        if (this.locals.containsKey(regionName)) {
            throw new IllegalArgumentException("Region with name '" + regionName + "' already exists in dimension '" + this.dim.getName() + "'!");
        }
        IMarkableRegion currentRegion = this.locals.get(region.getName());
        IProtectedRegion parent = currentRegion.getParent();
        this.removeLocal(currentRegion);
        currentRegion.rename(regionName);
        this.addLocal(parent, currentRegion);
    }

    /**
     * Method to check if a region name is valid for a given dimension. <br>
     * A region name is valid if it matches the pattern and is not already used in the dimension.
     *
     * @param regionName the name of the region to be checked.
     * @return -1 if the region name is invalid, 0 if the region name is valid, 1 if the region name is already used in the dimension.
     */
    public int isValidRegionName(String regionName) {
        List<String> commandStrings = Arrays.stream(values()).map(CommandConstants::toString).collect(Collectors.toList());
        if (!regionName.matches(RegionArgumentType.VALID_NAME_PATTERN.pattern())
                || commandStrings.contains(regionName.toLowerCase())) {
            return -1;
        }
        if (this.locals.containsKey(regionName)) {
            return 1;
        }
        return 0;
    }

    public void addLocal(IProtectedRegion parent, IMarkableRegion child) {
        parent.addChild(child);
        this.locals.put(child.getName(), child);
    }

    public void addLocal(IMarkableRegion child) {
        this.dim.addChild(child);
        this.locals.put(child.getName(), child);
    }

    public void removeLocal(IMarkableRegion region) {
        if (this.hasLocal(region.getName())) {
            this.locals.remove(region.getName());
            if (region.getParent().getRegionType() == RegionType.DIMENSION) {
                region.getParent().removeChild(region);
            }
        }
    }

    public boolean hasLocal(String regionName) {
        return locals.containsKey(regionName);
    }

    public IMarkableRegion getLocal(String regionName) {
        return locals.get(regionName);
    }

    public void clearLocals() {
        this.locals.clear();
        this.dim.clearChildren();
    }

}
